@@ -157,14 +157,14 @@ function getMostRecentWindow(aType) {
   return wm.getMostRecentWindow(aType);
 }
 
-//@line 165 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 165 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
 
 // this returns the most recent non-popup browser window
 function getMostRecentBrowserWindow() {
   var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                      .getService(Components.interfaces.nsIWindowMediator);
 
-//@line 172 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 172 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
   var win = wm.getMostRecentWindow("navigator:browser", true);
 
   // if we're lucky, this isn't a popup, and we can just return this
@@ -177,7 +177,7 @@ function getMostRecentBrowserWindow() {
         win = nextWin;
     }
   }
-//@line 197 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 197 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
 
   return win;
 }
@@ -328,6 +328,8 @@ var nsBrowserContentHandler = {
       openPreferences();
       cmdLine.preventDefault = true;
     }
+    if (cmdLine.handleFlag("silent", false))
+      cmdLine.preventDefault = true;
   },
 
   helpInfo : "  -browser            Open a browser window.\n",
@@ -631,9 +633,9 @@ var Module = {
 
     registerType("text/html");
     registerType("application/vnd.mozilla.xul+xml");
-//@line 651 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 653 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
     registerType("image/svg+xml");
-//@line 653 "/builds/tinderbox/Fx-Mozilla1.8.0/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
+//@line 655 "/builds/tinderbox/Fx-Mozilla1.8.0-Release/Linux_2.4.21-37.EL_Depend/mozilla/browser/components/nsBrowserContentHandler.js"
     registerType("text/rdf");
     registerType("text/xml");
     registerType("application/xhtml+xml");
